#!/bin/sh

if [ -z "${OSSPI_SCAN_ENABLED}" ] || [ "${OSSPI_SCAN_ENABLED}" = false ]; then
    printf "\e[0;31m%-6s\e[m\n" "** ERROR: OSSPI scan not enabled or environment variable is missing. OSSPI will not run .."
    exit 0
fi

# Use this to parse rules_osspi.yaml as json and the jq command to parse
# further
yaml2json() {
  python3 -c 'import sys, yaml, json; print(json.dumps(yaml.safe_load(sys.stdin.read())))'
}

# Display binary version
osspi --version

if [ -z "${OSSPI_API_KEY}" ] || [ -z "${OSM_API_KEY}" ] || [ -z "${OSM_USERNAME}" ]; then
    printf "\e[0;31m%-6s\e[m\n" "** ERROR: OSSPI or OSM API key not provided. These scans will not run. Gracefully exiting .."
    exit 0
fi

PHOTON_CT_TRACKER=4866992
CENTOS_CT_TRACKER=4866994
UBUNTU_CT_TRACKER=4866993
BASEOS_CT_TRACKER=0
OSM_INTERACTIONS="Distributed - Dynamic Link w/ VMW,Distributed - Dynamic Link w/ TP,Distributed - Dynamic Link w/ OSS"
OSM_API_FILE="osm_api.txt"

touch $OSM_API_FILE
echo "${OSM_USERNAME} ${OSM_API_KEY}" > $OSM_API_FILE

if [ -z "${OSSPI_PRODUCT_NAME}" ]; then
    OSSPI_PRODUCT_NAME="CarbonBlackCloud-Platform"
fi

if [ -z "${OSM_PRODUCT_NAME}" ]; then
    OSM_PRODUCT_NAME="CarbonBlackCloud-Platform/latest"
fi

if [ -z "${OSSPI_SERVER}" ]; then
    OSSPI_SERVER="https://osspi.eng.vmware.com"
fi

if [ -z "${OSM_SERVER}" ]; then
    OSM_SERVER="production"
fi

SCAN_STATUS=0

parallel_scan() {
    IMAGE_REGISTRY_NAME=$(echo "${IMAGE}" | cut -d'/' -f 1)
    IMAGE_REPO_NAME=$(echo "${IMAGE}" | cut -d'/' -f 2- | cut -d':' -f1 | sed 's:/:_:g')
    IMAGE_TAG=$(echo "${IMAGE}" | rev | cut -d":" -f1 | rev)
    OSSPI_FNAME="${CI_PIPELINE_ID}__${JENKINS_BUILD_ID}__${IMAGE_REGISTRY_NAME}__${IMAGE_REPO_NAME}__${IMAGE_TAG}"

    # shellcheck disable=SC3043,SC2155
    local random_str="$(tr -dc A-Za-z0-9 < /dev/urandom | head -c 10 ; echo '')"
    # shellcheck disable=SC3043,SC2155
    local osspi_tmpdir="tmpdir_${random_str}"

    SKIP=0
    for e in $(yaml2json <rules_osspi.yaml | jq '.exclusions[].regex' -r)
    do
        if echo "${IMAGE}" | grep "${e}"
        then
            echo -e "\e[1;33m" "** INFO: Skipping image due to the following regex: ${e}" "\e[0m"
            SKIP=1
            break
        fi
    done

    if test ${SKIP} -eq 1
    then
        return
    fi

    if ! docker pull "${IMAGE}"; then
        printf "\e[0;31m%-6s\e[m\n" "** ERROR: Failed pulling ${IMAGE}. Gracefully exiting .."
        exit 0
    fi

    docker save --output "${OSSPI_FNAME}.tar" "${IMAGE}"

    OS_NAME=$(docker run --rm --entrypoint cat "${IMAGE}" /etc/os-release | grep -i "^name=" | cut -d "=" -f 2 | tr "[:upper:]" "[:lower:]")
    printf "\e[1;32m%-6s\e[m\n" "** INFO: OS type detected as: ${OS_NAME}"

    if echo "${OS_NAME}" | grep -qi "photon"; then
        BASEOS_CT_TRACKER=$PHOTON_CT_TRACKER
    elif echo "${OS_NAME}" | grep -qi "centos"; then
        BASEOS_CT_TRACKER=$CENTOS_CT_TRACKER
    elif echo "${OS_NAME}" | grep -qi "ubuntu"; then
        BASEOS_CT_TRACKER=$UBUNTU_CT_TRACKER
    fi
    
    printf "\e[1;32m%-6s\e[m\n" "** INFO: Silently performing OSSPI scan and generating the manifest file for ${IMAGE}. Output will be provided at the end of the scan"
    # Using --conceal_metrics here to disable OSSPI scan metrics
    if ! osspi --conceal_metrics scan docker --image-tar "${OSSPI_FNAME}.tar" --format manifest --output-dir "${osspi_tmpdir}" > docker_scan.out 2>&1; then
        printf "\e[0;31m%-6s\e[m\n" "** ERROR: Failed performing OSSPI scan and generating the manifest file for ${IMAGE}. Gracefully exiting .."
        cat docker_scan.out
        exit 0
    fi

    for f in $(find "${osspi_tmpdir}" -maxdepth 1 -mindepth 1 -type f -exec readlink -f {} \;); do
        scanner=''

        if echo "${f}" | grep -qi "docker" && echo "${f}" | grep -qi ".manifest"; then
            scanner="osspi"

            printf "\e[1;32m%-6s\e[m\n" "** INFO: Silently uploading the OSSPI scan results for ${IMAGE} to the OSSPI dashboard. Output will be provided at the end of the scan"
            if ! osspi upload --source "${f}" --osspi-url "${OSSPI_SERVER}" --apikey "${OSSPI_API_KEY}" --product "${OSSPI_PRODUCT_NAME}" --nowait > osspi_upload.out 2>&1; then
                printf "\e[0;31m%-6s\e[m\n" "** ERROR: Failed uploading the OSSPI scan results for ${IMAGE} to the OSSPI dashboard. Gracefully exiting .."
                cat osspi_upload.out
                exit 0
            fi

            printf "\e[1;32m%-6s\e[m\n" "** INFO: Silently uploading the manifest file for ${IMAGE} to the OSM dashboard. Output will be provided at the end of the scan"
            if ! osstp -gn "${IMAGE_REPO_NAME}" -gv "${IMAGE_TAG}" -R "${OSM_PRODUCT_NAME}" -A "${OSM_API_FILE}" -S "${OSM_SERVER}" -F --noinput -I "${OSM_INTERACTIONS}" "${f}" > osm_upload.out 2>&1; then
                printf "\e[0;31m%-6s\e[m\n" "** ERROR: Failed uploading the manifest file for ${IMAGE} to the OSM dashboard (application packages). Gracefully exiting .."
                cat osm_upload.out
                exit 0
            fi
            
            printf "\e[1;32m%-6s\e[m\n" "** INFO: BaseOS CT Tracker selected: ${BASEOS_CT_TRACKER}"
            if ! osstp -R "${OSM_PRODUCT_NAME}" -A "${OSM_API_FILE}" -S "${OSM_SERVER}" --baseos-ct-tracker "${BASEOS_CT_TRACKER}" -F --noinput -I "${OSM_INTERACTIONS}" "${f}" > osm_upload2.out 2>&1; then
                printf "\e[0;31m%-6s\e[m\n" "** ERROR: Failed uploading the manifest file for ${IMAGE} to the OSM dashboard (OS packages). Gracefully exiting .."
                cat osm_upload2.out
                exit 0
            fi

            file_name="psec__${scanner}__${OSSPI_FNAME}.manifest"
            mv "${f}" "${file_name}"

            python3 analyser/osspiDockerScanNormalizer.py -fn "${file_name}" -in "${IMAGE}" -s "${scanner}"
        elif echo "${f}" | grep -qi "syft" || echo "${f}" | grep -qi "grype" || echo "${f}" | grep -qi "anchore"; then
            scanner=$(echo "$f" | sed 's:-:_:g' | cut -d '_' -f 1)
            mv "${f}" "${REP_PATH_CLI}/osspi__${scanner}__${OSSPI_FNAME}.json"
        fi

    done

    LAST_STATUS=$?

    # If this scan or a previous scan failed, keep the status
    if [ ${SCAN_STATUS} -ne 0 ] || [ ${LAST_STATUS} -ne 0 ]; then
        SCAN_STATUS=1
    fi

    printf "\e[1;32m%-6s\e[m\n" "**************** INFO: OSSPI docker scan output for ${IMAGE} ****************"
    cat docker_scan.out
    
    printf "\e[1;32m%-6s\e[m\n" "**************** INFO: OSSPI upload output for ${IMAGE} ****************"
    cat osspi_upload.out
    
    printf "\e[1;32m%-6s\e[m\n" "**************** INFO: OSM upload output for ${IMAGE} (application packages) ****************"
    cat osm_upload.out

    printf "\e[1;32m%-6s\e[m\n" "**************** INFO: OSM upload output for ${IMAGE} (OS packages) ****************"
    cat osm_upload2.out
}

printf "\e[1;32m%-6s\e[m\n" "** INFO: Updating grype vulnerabilities database .."
/root/.osspi/osspi/bin/grype db update

for IMAGE in ${SCAN_IMAGE_LIST}; do
    parallel_scan "${IMAGE}" &
    sleep 10
done

wait

printf "\e[1;32m%-6s\e[m\n" "** INFO: To access the OSSPI dashboard, please visit https://osspi.eng.vmware.com/audit-reports/all?limit=25&offset=0&id=13346&ordering=-recent_activity_time"
printf "\e[1;32m%-6s\e[m\n" "** INFO: To access the OSM dashboard, please visit https://osm.eng.vmware.com/oss/#/release/25529/packages"

rm -f "${OSM_API_FILE}" docker_scan.out osspi_upload.out osm_upload.out

exit ${SCAN_STATUS}